﻿// <copyright file="OclEnsureObjects.cs" company="Codeplex project Eco.Contrib"> 
// The MIT License (MIT) 
// </copyright> 
// <author>Peter Buchmann</author> 

namespace Eco.Contrib
{
    using Eco.ObjectRepresentation;
    using Eco.Ocl.Support;

    /// <summary>
    /// OCL function EnsureObjects
    /// </summary>
    [EcoOclOperation(typeof(OclEnsureObjects), true)]
    public class OclEnsureObjects : OclOperationBase
    {
        /// <summary>
        /// Implementation of OCL funktion EnsureObjects
        /// </summary>
        /// <param name="oclParameters">The parameters.</param>
        public override void Evaluate(IOclOperationParameters oclParameters)
        {
            if (oclParameters == null)
            {
                throw new EcoContribException("Parameter is null");
            }

            IElementCollection coll = Support.GetAsList(oclParameters.Values[0]);
            coll.EnsureRange(0, coll.Count - 1);
            oclParameters.Result.SetReferenceElement(coll);
        }

        /// <summary>
        /// Initialisation of OCL function EnsureObjects
        /// </summary>
        protected override void Init()
        {
            IOclType[] parameter = new IOclType[] { Support.ListType };
            InternalInit("EnsureObjects", parameter, OclResultTypeDeduceMethod.TypeOfSource);
        }
    }
}
